@@ -8,6 +8,7 @@
 -- Initialize some very basic things
 print = minetest.debug
 math.randomseed(os.time())
+os.setlocale("C", "numeric")
 
 -- Load other files
 dofile(minetest.get_modpath("__builtin").."/serialize.lua")
@@ -22,4 +23,6 @@ dofile(minetest.get_modpath("__builtin").."/auth.lua")
 dofile(minetest.get_modpath("__builtin").."/chatcommands.lua")
 dofile(minetest.get_modpath("__builtin").."/static_spawn.lua")
 dofile(minetest.get_modpath("__builtin").."/detached_inventory.lua")
+dofile(minetest.get_modpath("__builtin").."/falling.lua")
+dofile(minetest.get_modpath("__builtin").."/features.lua")
 
